@@ -1,8 +1,4 @@
-idc <-
-function(y, x, z, P, G, to, tree) {
-  G.obs <- observed.graph(G)
-  v <- get.vertex.attribute(G, "name")
-  v <- to[which(to %in% v)]
+idc <- function(y, x, z, P, G, G.obs, v, to, tree) {
   if (length(P$var) == 0) tree$call <- list(y = y, x = x, z = z, P = probability(var = v), G = G, line = "", v = v)
   else tree$call <- list(y = y, x = x, z = z, P = P, G = G, line = "", v = v)
   tree$branch <- list()
@@ -17,13 +13,13 @@ function(y, x, z, P, G, to, tree) {
     if (dSep(A, y, node, union(x, cond))) {
       tree$call$line <- 9
       tree$call$z.prime <- node
-      nxt <- idc(y, union(x, node), cond, P, G, to, list())
+      nxt <- idc(y, union(x, node) %ts% to, cond, P, G, G.obs, v, to, list())
       tree$branch[[1]] <- nxt$tree
       return(list(P = nxt$P, tree = tree))
     } 
   }
   tree$call$line <- 10
-  nxt <- id(union(y, z), x, P, G, to, list())
+  nxt <- id(union(y, z) %ts% to, x, P, G, G.obs, v, to, list())
   tree$branch[[1]] <- nxt$tree
   return(list(P = nxt$P, tree = tree))
 }
